@@ -37,6 +37,7 @@ VALUE mCORBA;
 VALUE eComError;
 VALUE eCORBA;
 VALUE eCORBAComError;
+VALUE eCORBATimeoutError;
 VALUE cNameService;
 VALUE cTaskContext;
 VALUE eNotFound;
@@ -324,7 +325,7 @@ static VALUE task_context_port_names(VALUE self)
 //
 // Returns the state of the task, as an integer value. The possible values are
 // represented by the various +STATE_+ constants:
-// 
+//
 //   STATE_PRE_OPERATIONAL
 //   STATE_STOPPED
 //   STATE_ACTIVE
@@ -435,7 +436,7 @@ static RTT::corba::CConnPolicy policyFromHash(VALUE options)
 }
 
 /* Actual implementation of #connect_to. Sanity checks are done in Ruby. Just
- * create the connection. 
+ * create the connection.
  */
 static VALUE do_port_connect_to(VALUE routput_port, VALUE rinput_port, VALUE options)
 {
@@ -616,6 +617,7 @@ extern "C" void Init_rorocos()
     eComError    = rb_define_class_under(mOrocos, "ComError", rb_eRuntimeError);
     eCORBA    = rb_define_class_under(mOrocos, "CORBAError", rb_eRuntimeError);
     eCORBAComError = rb_define_class_under(mCORBA, "ComError", eComError);
+    eCORBATimeoutError = rb_define_class_under(mCORBA, "TimeoutError", eCORBA);
     eNotInitialized = rb_define_class_under(mOrocos, "NotInitialized", rb_eRuntimeError);
     eBlockingCallInForbiddenThread = rb_define_class_under(mOrocos, "BlockingCallInForbiddenThread", rb_eRuntimeError);
 
