@@ -130,10 +130,18 @@ module Orocos
         def self.load_raw_sections_from_file(file)
             document_lines = File.readlines(file)
 
-            headers = document_lines.each_with_index.
-                find_all { |line, _| line =~ /^---/ }
-            if headers.empty? || headers.first[1] != 0
+            headers =
+                document_lines
+                .each_with_index
+                .find_all { |line, _| line =~ /^---/ }
+
+            if headers.empty?
                 headers.unshift ["--- name:default", -1]
+            elsif headers.first[1] != 0
+                leading_lines = document_lines[0, headers.first[1]].map(&:strip)
+                if leading_lines.any? { |l| !l.empty? && !l.start_with?("#") }
+                    headers.unshift ["--- name:default", -1]
+                end
             end
 
             options = headers.map do |line, line_number|
@@ -171,7 +179,16 @@ module Orocos
             end
             sections << document_lines[options[-1][1] + 1, document_lines.size - options[-1][1] - 1]
 
-            options.map(&:first).zip(sections)
+            sections = options.map(&:first).zip(sections)
+            found_sections = []
+            sections.each do |conf_options, doc|
+                name = conf_options[:name]
+                if found_sections.include?(name)
+                    raise ArgumentError, "#{name} defined twice"
+                end
+                found_sections << name
+            end
+            sections
         end
 
         # @api private
@@ -246,9 +263,10 @@ module Orocos
                     changed_sections << name
                 end
             end
-	    if !changed_sections.empty?
-	    	@merged_conf.clear
-	    end
+
+            if !changed_sections.empty?
+                @merged_conf.clear
+            end
             changed_sections
         rescue Exception => e
             raise e, "error loading #{file}: #{e.message}", e.backtrace
@@ -797,13 +815,13 @@ module Orocos
             if value.kind_of?(Typelib::ArrayType)
                 # This is a fixed-size array, verify that the size matches
                 if conf.size > value.size
-                    raise ArgumentError, "Configuration object size is larger than field #{value}"
+                    raise ArgumentError,
+                          "Configuration object size is larger than field #{value}"
                 end
             else
                 element_t = value.class.deference
                 while value.size < conf.size
-                    new_value = element_t.new
-                    new_value.zero!
+                    new_value = element_t.zero
                     value.push(new_value)
                 end
             end
@@ -918,7 +936,7 @@ module Orocos
         # @overload save(section_name, file, replace: false, task_model: self.model)
         #   @param [String] section_name the section name
         #   @param [String] file either a file, or a directory. In the latter
-        #     case, the file will be #{conf_dir}/#{model.name}.yml
+        #     case, the file will be #\\{conf_dir\}/#\\{model\.name\}.yml
         #   @return [Hash] the configuration section that just got saved
         #
         # @overload save(task, file, section_name)
